import csv


def clean_csv(data: list[list]):
    """
    Removes trailing empty strings from CSVs that ovvur when extra commas exist
    
    :param data: list of lists returned from from_csv function
    :type data: list of lists
    :rtype: list[list]
    """
    while data[0][-1] == "":
        for _ in data:
            _.pop()
    return data

def from_csv(file: str) -> list[list]:

    """
    Read CSV file into a list of lists.

    Takes a CSV file path, opens the file, and returns the 
    contents parsed into a list of rows, where each row 
    is a list of the data cells.

    :param file: Path to CSV file
    :type file: str
    :returns: Parsed CSV data
    :rtype: list[list]
    """

    with open(file) as f:

        csv_file = csv.reader(f)
        data = [row for row in csv_file]
    return clean_csv(data)



from toRST.toRST import Table
from csv2rst.csv2rst import clean_csv
from funcs.funcs import handle_raw
import pytest


@pytest.fixture
def empty_table():

    yield Table('tests/csv/empty.csv')

@pytest.fixture
def large_table():

    yield Table('tests/csv/large.csv')

@pytest.fixture
def small_table():

    yield Table('tests/csv/small.csv')

def test_headers(empty_table, large_table, small_table) -> None:

    assert empty_table.headers == ["No","Data","Here"]
    assert large_table.headers == ["First Name","Last Name","Job Title","Biography"]
    assert small_table.headers == ["Header1","Header2","Header3"]

def test_column_widths(empty_table, large_table, small_table) -> None:

    assert empty_table.column_widths[0] == len("No")
    assert large_table.column_widths[1] == len("Livingston")
    assert small_table.column_widths[2] == len("Header3")

def test_clean_csv() -> None:

    assert clean_csv([["ab", "cd", "ef", "gh", "", ""], ["ba", "dc", "fe", "hg", "", ""]]) == [["ab", "cd", "ef", "gh"], ["ba", "dc", "fe", "hg"]]

def test_handle_raw() -> None:

    with pytest.raises(ValueError) as value_error:
        handle_raw({"key1": "value1", "key2": "value2"})
    assert str(value_error.value) == "input must be of type list[list or dict or tuple]"

    with pytest.raises(ValueError) as value_error:
        handle_raw(["value", "value", "value", "value"])
    assert str(value_error.value) == "input must be of type list[list or dict or tuple]"



import json


def from_json(file) -> list[list]:

    data=json.load(open(file, "r"))
    if isinstance(data, dict):
        raise ValueError("JSON must be list of lists or list of dicts")
    elif isinstance(data, list):
        if isinstance(data[0], list):
            return data
        else:
            return [[key for key in data[0].keys()]] + [list(_.values()) for _ in data]


from csv2rst.csv2rst import from_csv
from json2rst.json2rst import from_json


def get_extension(file: str) -> str:

    return file.split(".")[-1].lower()

def handle_file(file: str) -> list[list]:

    extension = get_extension(file)
    if extension == "csv":
        return from_csv(file)
    elif extension == "json":
        return from_json(file)
    else:
        raise ValueError(f"{extension} files not currently supported")

def handle_raw(input: list[list or dict]) -> list[list]:

    if not isinstance(input, list):
        raise ValueError("input must be of type list[list or dict or tuple]")
    else:
        if isinstance(input[0], list):
            return input
        elif isinstance(input[0], dict):
            return [[key for key in input[0].keys()]] + [list(row.values()) for row in input[1:]]
        elif isinstance(input[0], tuple):
            return [list(row) for row in input]
        else: raise ValueError("input must be of type list[list or dict or tuple]")


import argparse


parser = argparse.ArgumentParser(
    description="""
    =====================
    CSV to RST converter
    =====================

    Converts CSV files to reStructuredText grids.

    Positional Arguments:
    csv_files               One or more CSV files to convert  

    Optional Arguments:
    -o, --output_dir        Output directory for RST files

    Example usage:

    torst mydata.csv mydata2.csv -o output/

    This will generate mydata.rst and mydata2.rst in the output folder.
    """
)
parser.add_argument(
    "files", nargs="+", help="""
    Positional Arguments
    --------------------

    files
    One or more files to convert. Multiple files can be specified separated  
    by spaces. The path can be absolute or relative to the current working
    directory.

    - *Type:* String
    - *Required:* Yes
    """
)

parser.add_argument(
    "-o", "--output_dir", default=".", help="""
    Optional Arguments:

    -o, --output_dir
    Output directory for generated RST files. Defaults to the current 
    working directory if not provided.
    """
)

def get_file_name(file: str) -> str:

    return file.split("/")[-1].split(".")[0]



from setuptools import setup, find_packages


readme = """
==========================================================
<br>toRST
=====

Convert various data formats to reStructuredText tables.

**Currently supports**:
----------------------

- CSV
- JSON

**Planned formats**:


- Excel<br><br>

**installation**
```bash
pip install toRST
```
<br>

**CLI Usage**
-------------
example<br>
  Convert file1.csv and file2.json into RST
```bash
torst file1.csv file2.json -o ./outputfolder
```

**Positional Arguments**:
------------------------

inputs
  One or more input files to convert to RST. Currently only CSV and JSON files are 
  supported, but additional formats will be added.

**Optional Arguments**:
---------------------- 

-o, --output_dir
  Output directory for generated RST files. Defaults to the current 
  working directory if not provided.

**Python Usage**
-------------
example<br>
  
Import Table class
```python
from toRST.toRST import Table
```
Convert file1.csv into RST string
```python
rst_table = Table('file1.csv').build_table()
```
Can also convert ```list[list or tuple or dict]``` into RST by passing the object into the ```Table``` class<br> 
""".strip()

with open('LICENSE') as f:
    license = f.read()
setup(
    name='toRST',
    version='0.1.0',
    description='Command line tool for converting CSV and JSON files into reStructuredText Tables.',
    long_description=readme,
    author='John Reynolds',
    author_email='reynoldsjohngreg@gmail.com',
    url='https://github.com/jrey999/toRST',
    license=license,
    packages=find_packages(exclude=('tests',)),
    entry_points={
        'console_scripts': [
            'torst = toRST.write:torst'
        ]
    }
)


from toRST.toRST import Table
from funcs.command import parser, get_file_name


args = parser.parse_args()
def torst():
  
  for file in args.files:

    with open(f"{args.output_dir}/{get_file_name(file)}.rst", "w") as f:
        
        f.write(Table(file).build_table())
        f.close()

if __name__ == "__main__":
    torst()